/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│ vi: set et ft=c ts=2 sts=2 sw=2 fenc=utf-8                               :vi │
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2022 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/calls/calls.h"
#include "libc/macros.h"
#include "libc/runtime/runtime.h"

/**
 * Copies data between file descriptors the old fashioned way.
 *
 * This function is intended for simple programs without signals. If
 * signals are in play, then `SA_RESTART` needs to be used.
 *
 * @param in is input file descriptor
 * @param out is input file descriptor
 * @param n is number of bytes to exchange, or -1 for until eof
 * @return bytes successfully exchanged, or -1 w/ errno
 */
ssize_t copyfd(int in, int out, size_t n) {
  size_t i;
  char buf[512];
  ssize_t dr, dw;
  for (i = 0; i < n; i += dr) {
    dr = read(in, buf, MIN(n - i, sizeof(buf)));
    if (dr == -1)
      return -1;
    if (!dr)
      break;
    dw = write(out, buf, dr);
    if (dw == -1)
      return -1;
    if (dw != dr) {
      // POSIX requires atomic IO up to PIPE_BUF
      // The minimum permissible PIPE_BUF is 512
      notpossible;
    }
  }
  return i;
}
